// ==============================================================
//                 ORBITER MODULE: Atlantis
//                  Part of the ORBITER SDK
//            Copyright (C) 2001 Martin Schweiger
//                   All rights reserved
//
// Common.cpp
// Some utility function common to multiple Atlantis-related
// modules
// ==============================================================

#include "Atlantis.h"

int SRB_nt = 6;
double SRB_Seq[6]    = {0,SRB_STABILISATION_TIME,    104,    116,SRB_SEPARATION_TIME,136};
double SRB_Thrust[6] = {0,                     1,      1,   0.85,               0.05,  0};
double SRB_Prop[6]   = {1,               0.99583,0.13588,0.04321,           0.001879,  0};
double SRB_ThrSCL[5] = {(SRB_Thrust[1]-SRB_Thrust[0])/(SRB_Seq[1]-SRB_Seq[0]),
						(SRB_Thrust[2]-SRB_Thrust[1])/(SRB_Seq[2]-SRB_Seq[1]),
						(SRB_Thrust[3]-SRB_Thrust[2])/(SRB_Seq[3]-SRB_Seq[2]),
						(SRB_Thrust[4]-SRB_Thrust[3])/(SRB_Seq[4]-SRB_Seq[3]),
						(SRB_Thrust[5]-SRB_Thrust[4])/(SRB_Seq[5]-SRB_Seq[4])};
double SRB_PrpSCL[5] = {(SRB_Prop[1]-SRB_Prop[0])/(SRB_Seq[1]-SRB_Seq[0]),
						(SRB_Prop[2]-SRB_Prop[1])/(SRB_Seq[2]-SRB_Seq[1]),
						(SRB_Prop[3]-SRB_Prop[2])/(SRB_Seq[3]-SRB_Seq[2]),
						(SRB_Prop[4]-SRB_Prop[3])/(SRB_Seq[4]-SRB_Seq[3]),
						(SRB_Prop[5]-SRB_Prop[4])/(SRB_Seq[5]-SRB_Seq[4])};

// time-dependent calculation of SRB thrust and remaining propellant
void GetSRB_State (double srb_time, double &thrust_level, double &prop_level)
{
	for (int i = SRB_nt-2; i >= 0; i--)
		if (srb_time > SRB_Seq[i]) break;
	thrust_level = SRB_ThrSCL[i] * (srb_time-SRB_Seq[i]) + SRB_Thrust[i];
	prop_level = SRB_PrpSCL[i] * (srb_time-SRB_Seq[i]) + SRB_Prop[i];
}

