#define ORBITER_MODULE
#include "Orbitersdk.h"
#include "resource.h"
#include <stdio.h>

// Global parameters
struct {
	HINSTANCE hDLL;
	HWND      hDlg;
	HWND      renderWnd;
	OBJHANDLE hVessel;
	int       mainretro_slider_pos;
	int       hover_slider_pos;
} g_Param;

BOOL CALLBACK DlgProc (HWND, UINT, WPARAM, LPARAM);
void CreateVesselList (void);

// DLL initialisation
DLLCLBK void opcDLLInit (HINSTANCE hDLL)
{
	g_Param.hDLL = hDLL;
}

// Simulation initialisation
DLLCLBK void opcOpenRenderViewport (HWND renderWnd, DWORD width, DWORD height, BOOL fullscreen)
{
	g_Param.renderWnd = renderWnd;

	if (fullscreen) { // don't open external windows in fullscreen mode
		g_Param.hDlg = 0;
	} else {          // create the plugin interface dialog
		g_Param.hDlg = CreateDialog (g_Param.hDLL, MAKEINTRESOURCE(IDD_INTERFACE), 0, (DLGPROC)DlgProc);
		CreateVesselList ();
		SetScrollRange (GetDlgItem (g_Param.hDlg, IDC_MAIN_SLIDER), SB_CTL, 0, 100, FALSE);
		SetScrollRange (GetDlgItem (g_Param.hDlg, IDC_RETRO_SLIDER), SB_CTL, 0, 100, FALSE);
		SetScrollRange (GetDlgItem (g_Param.hDlg, IDC_HOVER_SLIDER), SB_CTL, 0, 100, FALSE);
		g_Param.mainretro_slider_pos = g_Param.hover_slider_pos = 0;
	}
}

// Simulation cleanup
DLLCLBK void opcCloseRenderViewport ()
{
	if (g_Param.hDlg) {
		DestroyWindow (g_Param.hDlg);
		g_Param.hDlg = 0;
	}
}

// Frame update
DLLCLBK void opcTimestep (double SimT, double SimDT, double mjd)
{
	if (!g_Param.hDlg) return;

	ENGINESTATUS es;
	int slider;
	oapiGetEngineStatus (g_Param.hVessel, &es);
	slider = (int)(es.main*100.0);
	if (slider != g_Param.mainretro_slider_pos) {
		if (slider >= 0) {
			SetScrollPos (GetDlgItem (g_Param.hDlg, IDC_MAIN_SLIDER), SB_CTL, slider, TRUE);
			SetScrollPos (GetDlgItem (g_Param.hDlg, IDC_RETRO_SLIDER), SB_CTL, 0, TRUE);
		} else {
			SetScrollPos (GetDlgItem (g_Param.hDlg, IDC_MAIN_SLIDER), SB_CTL, 0, TRUE);
			SetScrollPos (GetDlgItem (g_Param.hDlg, IDC_RETRO_SLIDER), SB_CTL, -slider, TRUE);
		}
		g_Param.mainretro_slider_pos = slider;
	}
	slider = (int)(es.hover*100.0);
	if (slider != g_Param.hover_slider_pos) {
		SetScrollPos (GetDlgItem (g_Param.hDlg, IDC_HOVER_SLIDER), SB_CTL, slider, TRUE);
		g_Param.hover_slider_pos = slider;
	}
}

// Create vessel list
void CreateVesselList (void)
{
	if (!g_Param.hDlg) return;

	char cbuf[128];
	DWORD i, n = oapiGetVesselCount();
	SendDlgItemMessage (g_Param.hDlg, IDC_VESSELLIST, CB_RESETCONTENT, 0, 0);
	for (i = 0; i < n; i++) {
		OBJHANDLE hVessel = oapiGetVesselByIndex (i);
		oapiGetObjectName (hVessel, cbuf, 128);
		SendDlgItemMessage (g_Param.hDlg, IDC_VESSELLIST, CB_ADDSTRING, 0, (LPARAM)cbuf);
	}
	g_Param.hVessel = oapiGetFocusObject();
	oapiGetObjectName (g_Param.hVessel, cbuf, 128);
	SendDlgItemMessage (g_Param.hDlg, IDC_VESSELLIST, CB_SELECTSTRING, 0, (LPARAM)cbuf);
}

// Dialog message callback function
BOOL CALLBACK DlgProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if (!g_Param.hDlg) return 0;

	char cbuf[128];
	int idx, sliderpos;
	OBJHANDLE hVessel;
	ENGINETYPE et;

	switch (uMsg) {
	case WM_INITDIALOG:
		//CreateVesselList ();
		return 0;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_FOCUS:
			oapiSetFocusObject (g_Param.hVessel);
			break;
		case IDC_VESSELLIST:
			if (HIWORD(wParam) == CBN_SELCHANGE) {
				idx = SendDlgItemMessage (g_Param.hDlg, IDC_VESSELLIST, CB_GETCURSEL, 0, 0);
				SendDlgItemMessage (g_Param.hDlg, IDC_VESSELLIST, CB_GETLBTEXT, idx, (LPARAM)cbuf);
				hVessel = oapiGetVesselByName (cbuf);
				if (hVessel) g_Param.hVessel = hVessel;
			}
			break;
		}
	case WM_HSCROLL:
		if ((HWND)lParam == GetDlgItem (g_Param.hDlg, IDC_MAIN_SLIDER))
			et = ENGINE_MAIN;
		else if ((HWND)lParam == GetDlgItem (g_Param.hDlg, IDC_RETRO_SLIDER))
			et = ENGINE_RETRO;
		else if ((HWND)lParam == GetDlgItem (g_Param.hDlg, IDC_HOVER_SLIDER))
			et = ENGINE_HOVER;
		else break;
		switch (LOWORD(wParam)) {
		case SB_LINEUP:
		case SB_LINEDOWN:
		case SB_THUMBPOSITION:
			sliderpos = HIWORD(wParam);
			oapiSetEngineLevel (g_Param.hVessel, et, sliderpos*0.01);
			break;
		}
	}
	return 0;
}