#define STRICT
#define ORBITER_MODULE
#include "Orbitersdk.h"
#include "resource.h"
#include <stdio.h>
#include <math.h>

// Global parameters
struct {
	HINSTANCE hDLL;
	HWND      hDlg;
	HWND      renderWnd;
	double    warpMin;
	double    warpMax;
	double    warpMinLog;
	double    warpRangeLog;
	int       warpSlider;
} g_Param;

BOOL CALLBACK DlgProc (HWND, UINT, WPARAM, LPARAM);
void EchoWarp (double warp);

// DLL initialisation
DLLCLBK void opcDLLInit (HINSTANCE hDLL)
{
	g_Param.hDLL = hDLL;
}

// Simulation initialisation
DLLCLBK void opcOpenRenderViewport (HWND renderWnd, DWORD width, DWORD height, BOOL fullscreen)
{
	g_Param.renderWnd = renderWnd;

	if (fullscreen) { // don't open external windows in fullscreen mode
		g_Param.hDlg = 0;
	} else {          // create the plugin interface dialog
		g_Param.hDlg = CreateDialog (g_Param.hDLL, MAKEINTRESOURCE(IDD_WARPDIALOG), 0, (DLGPROC)DlgProc);
		g_Param.warpMin = 1.0;
		g_Param.warpMax = 1000.0;
		g_Param.warpMinLog = log10 (g_Param.warpMin);
		g_Param.warpRangeLog = log10 (g_Param.warpMax) - g_Param.warpMinLog;
		SetScrollRange (GetDlgItem (g_Param.hDlg, IDC_WARP_SLIDER), SB_CTL, 0, 100, FALSE);
		EchoWarp (oapiGetTimeAcceleration());
	}
}

// Simulation cleanup
DLLCLBK void opcCloseRenderViewport ()
{
	if (g_Param.hDlg) {
		DestroyWindow (g_Param.hDlg);
		g_Param.hDlg = 0;
	}
}

// Respond to acceleration change
DLLCLBK void opcTimeAccChanged (double nWarp, double oWarp)
{
	if (g_Param.hDlg)
		EchoWarp (nWarp);
}

void EchoWarp (double warp)
{
	char cbuf[10], pcbuf[10];

	if (g_Param.hDlg) {
		sprintf (cbuf, "%0.0f", warp);
		GetWindowText (GetDlgItem (g_Param.hDlg, IDC_WARP), pcbuf, 10);
		if (strcmp (cbuf, pcbuf))
			SetWindowText (GetDlgItem (g_Param.hDlg, IDC_WARP), cbuf);
		g_Param.warpSlider = (int)(100.0 * (log10(warp)-g_Param.warpMinLog)/g_Param.warpRangeLog);
		SetScrollPos (GetDlgItem (g_Param.hDlg, IDC_WARP_SLIDER), SB_CTL, g_Param.warpSlider, TRUE);
	}
}

// Dialog message callback function
BOOL CALLBACK DlgProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	const double EPS = 1e-6;
	int sliderpos = g_Param.warpSlider;
	double warp;
	char cbuf[20];

	switch (uMsg) {
	case WM_INITDIALOG:
		return 1;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_WARP1:
			oapiSetTimeAcceleration (1.0);
			break;
		case IDC_WARP10:
			oapiSetTimeAcceleration (10.0);
			break;
		case IDC_WARP100:
			oapiSetTimeAcceleration (100.0);
			break;
		case IDC_WARP1000:
			oapiSetTimeAcceleration (1000.0);
			break;
		case IDC_WARP:
			if (HIWORD(wParam) == EN_CHANGE) {
				GetWindowText (GetDlgItem (g_Param.hDlg, IDC_WARP), cbuf, 10);
				if (sscanf (cbuf, "%lf", &warp) == 1)
					oapiSetTimeAcceleration (warp);
			}
			return 1; // keep focus to allow editing
		}
		SetFocus (g_Param.renderWnd);
		return 1;
	case WM_HSCROLL:
		switch (LOWORD(wParam)) {
		case SB_LINEUP:
			sliderpos = max (0, g_Param.warpSlider-1);
			warp = pow (10.0, 0.01*sliderpos*g_Param.warpRangeLog + g_Param.warpMinLog) - EPS;
			break;
		case SB_LINEDOWN:
			sliderpos = min (100, g_Param.warpSlider+1);
			warp = pow (10.0, 0.01*sliderpos*g_Param.warpRangeLog + g_Param.warpMinLog) + EPS;
			break;
		case SB_THUMBTRACK:
		case SB_THUMBPOSITION:
			sliderpos = HIWORD(wParam);
			warp = pow (10.0, 0.01*sliderpos*g_Param.warpRangeLog + g_Param.warpMinLog);
			break;
		}
		if (sliderpos != g_Param.warpSlider)
			oapiSetTimeAcceleration (warp);
		SetFocus (g_Param.renderWnd);
		return 1;
	}
	return 0;
}